#' Check for working HOMER installation
#'
#' A simple function used to check that HOMER is installed and working.
#'
#' @return logical; \code{TRUE} for success, \code{FALSE} otherwise
#' @export
check_homer <- function() {
    cmd <- "findMotifsGenome.pl"
    output <- system2(cmd, stdout = TRUE, stderr = TRUE)
    test_string <- "Program will find de novo and known motifs in regions"
    test_result <- stringr::str_detect(output[2], test_string)

    if (test_result == TRUE) {
        message("HOMER installation found")
    } else {
        warning("HOMER installation not found - check the HOMER website")
    }

    invisible(test_result)
}

#' Get the base installation directory of HOMER
#'
#' @return character vector with base directory of HOMER installation
#' @export
get_homer_base <- function() {
    cmd <- "which"
    args <- "findMotifsGenome.pl"
    homer_base <- system2(cmd, args, stdout = TRUE, stderr = TRUE) %>%
        dirname %>% dirname
    return(homer_base)
}


#' List installed/available HOMER packages
#'
#' Runs the configureHomer.pl script to get list of installed and
#' available packages, parsing the output into a tidy tibble.
#'
#' @param installed_only only show installed packages (default: TRUE)
#' 
#' @return \code{tibble} with status of installed (+) or not (-),
#'   the package, version, and description.
#' @export
list_homer_packages <- function(installed_only = TRUE) {
    homer_base <- get_homer_base()
    cmd <- "perl"
    args <- paste0(homer_base, "/configureHomer.pl ",
                   "-list")

    ## Parse output to remove header
    output <- system2(cmd, args, stdout = TRUE, stderr = TRUE) %>%
        stringr::str_replace_all("\t", " ") %>%
        .[stringr::str_detect(., "v[0-9]")]

    ## Parse each line further to generate tibble of packages
    .parse_out <- function(x) { x %>% t() %>% tibble::as_tibble() }
    parsed <- stringr::str_split(output, " ", n = 4) %>%
        purrr::map(.parse_out) %>%
        dplyr::bind_rows() %>%
        dplyr::rename_("status" = "V1",
                       "package" = "V2",
                       "version" = "V3",
                       "description" = "V4")

    ## Return either only installed or all packages
    if (installed_only == TRUE) {
        dplyr::filter_(parsed, "status == '+'") %>%
            return
    } else {
        return(parsed)
    }
}
