#' Read Known Enriched Motifs HOMER output
#'
#' Reads in the HOMER output describing enriched known motifs
#' into a tidy tibble.
#'
#' @param path path to the HOMER directory where all outputs are
#'   stored
#' @param homer_dir does the path point to a HOMER directory;
#'   if `FALSE`, path must point to the file directly
#'   [default: TRUE]
#'
#' @return a tibble with the following columns:
#' \itemize{
#'   \item \code{motif} the readable name of the sequence
#'   \item \code{family} of transcription factors the motif belongs to
#'   \item \code{experiment} from where the motif was identified
#'   \item \code{publication} source publication or online repository ID
#'   \item \code{database} the curator of the motif
#'   \item \code{consensus} the accepted consensus motif sequence
#'   \item \code{p_value} significance of enrichment
#'   \item \code{fdr} Benjaminni-Hochberg corrected p-value
#'   \item \code{tgt_num} number of times motif appears in target sequences
#'   \item \code{tgt_pct} percent of times motif appears in target sequences
#'   \item \code{bgd_num} number of times motif appears in background sequences
#'   \item \code{bgd_pct} percent of times motif appears in background sequences
#' }
#' 
#' @importFrom stringr str_replace
#' @importFrom lazyeval interp
#' @importFrom dplyr vars contains
#' 
#' @export

## path <- "inst/extdata"
read_known_results <- function(path, homer_dir = TRUE) {
    if (homer_dir == TRUE) {
        path <- paste0(path, "/knownResults.txt")
    }

    ## Read in raw file
    col_spec <- readr::cols('c', 'c', 'd', '-', 'd', 'd', 'c', 'd', 'c')
    raw <- readr::read_tsv(path, col_types = col_spec)
    colnames(results) <- c('motif_name', 'consensus',
                           'p_value', 'fdr',
                           'tgt_num', 'tgt_pct', 'bgd_num', 'bgd_pct')

    ## Parse down all the combined columns
    parsed <- results %>%
        tidyr::separate_("motif_name", c('motif', 'experiment', 'database'),
                        '/', extra = 'drop') %>%
        tidyr::separate_('motif', c('motif', 'family'), '\\(') %>%
        tidyr::separate_('experiment', c('experiment', 'publication'), '\\(') %>%
        dplyr::mutate_(
                   family = interp(
                       ~str_replace(var, '\\)', ''),
                       var = as.name("family")),
                   publication = interp(
                       ~str_replace(var, '\\)', ''),
                       var = as.name("publication"))
               ) %>%
        dplyr::mutate_at(vars(contains('pct')), .parse_pcts)

    return(parsed)
}

#' Read homerResults file - denovo motif enrichments
#'
#' Reads in the HOMER output describing enriched denovo motifs
#' into a tidy tibble.
#'
#' @param path path to the HOMER directory where all outputs are
#'   stored
#' @param homer_dir does the path point to a HOMER directory;
#'   if `FALSE`, path must point to the file directly
#'   [default: TRUE]
#'
#' @return a tibble with the following columns:
#' \itemize{
#'   \item \code{consensus} the consensus sequence of the denovo motif
#'   \item \code{motif_name} name of the motif
#'   \item \code{log_odds_detection_threshold} used to determine bound vs. unbound sites
#'   \item \code{log_pvalue_enrichment}
#'   \item \code{tgt_num} number of times motif appears in target sequences
#'   \item \code{tgt_pct} percent of times motif appears in target sequences
#'   \item \code{bgd_num} number of times motif appears in background sequences
#'   \item \code{bgd_pct} percent of times motif appears in background sequences
#'   \item \code{p_value} final enrichment p-value
#'   \item \code{tgt_pos} average position of motif in target sequences, where 0 = start of sequences
#'   \item \code{tgt_std} standard deviation of position in target sequences
#'   \item \code{bgd_pos} average position of motif in background sequences, where 0 = start of sequences
#'   \item \code{bgd_std} standard deviation of position in background sequences
#'   \item \code{strand_bias} log ratio of + strand occurences to - strand occurences
#'   \item \code{multiplicity} average number of occurences per sequence in sequences with 1 or more binding sites
#' }
#' 
#' @importFrom stringr str_replace str_split str_detect
#' @importFrom lazyeval interp
#' @importFrom dplyr vars contains select_ rename_ group_by group_by_ bind_cols
#' @importFrom dplyr mutate_ mutate_at mutate_all mutate 
#' @importFrom tidyr separate separate_ nest
#' @importFrom purrr map
#' @importFrom readr read_tsv
#' 
#' @export

## path <- "inst/extdata"

read_denovo_results <- function(path, homer_dir = TRUE) {
    if (homer_dir == TRUE) {
        path <- paste0(path, "/homerMotifs.all.motifs")
    }

    ## Read in raw file
    all <- suppressWarnings(read_tsv(path, col_names = FALSE))

    ## Calculate separation of each motif
    gt <- str_detect(all$X1, '>')

    ## Extract motif descriptors by filtering on '>'
    motif_info <- all[gt, ] %>%
        select_(interp(~-var, var = as.name("X5"))) %>%
        rename_(consensus = 'X1', motif_name = 'X2',
                       log_odds_detection_threshold = 'X3',
                       log_pvalue_enrichment = 'X4',
                       occurence = 'X6',
                       stats = 'X7') %>%
        mutate_(
                   consensus = interp(~str_replace(var, '>', ''),
                                                var = as.name("consensus"))
               )

    ## Helper formatting functions
    .drop_prior <- function(x, pattern = ":") {
        str_split(x, pattern) %>%
            purrr::map(function(x) { x[2] }) %>% unlist
    }
    .format_pct <- function(x) {
        str_replace(x, '%\\)', '') %>%
            as.numeric * 0.01
    }

    ## Munge motif position statistics
    stats <- motif_info[, 'stats'] %>%
        separate_('stats',
                         c('tgt_pos', 'tgt_std', 'bgd_pos', 'bgd_std',
                           'strand_bias', 'multiplicity'), ',') %>%
        mutate_all(.drop_prior) %>%
        mutate_all(as.numeric)

    ## Munge motif occurence metrics
    occurence <- motif_info[, 'occurence'] %>%
        separate_('occurence', c('tgt_num', 'bgd_num', 'p_value'), ',') %>%
        mutate_all(.drop_prior) %>%
        separate_('tgt_num', c('tgt_num', 'tgt_pct'), '\\(') %>%
        separate_('bgd_num', c('bgd_num', 'bgd_pct'), '\\(') %>%
        mutate_at(vars(contains('_pct')), .format_pct) %>%
        mutate_all(as.numeric)

    ## Munge motif PWMs
    ## Get motifs - where no '>' is detected
    ## Each motif instance is separated by a '>'
    ## Use cumsum to add up where '>' occur consecutively
    motif_pwm <- all[!gt, c('X1', 'X2', 'X3', 'X4')] %>%
        mutate(motif_id = cumsum(gt)[!gt]) %>%
        rename_(A = 'X1', C = 'X2', G = 'X3', T = 'X4') %>%
        group_by_('motif_id') %>%
        nest(.key = 'motif_pwm') %>%
        select_(interp(~-var, var = as.name('motif_id')))
    

    ## Put it all together
    motif_final <- motif_info %>%
        select_(interp(~-var,
                                        var = as.name('occurence'))) %>%
        select_(interp(~-var,
                                        var = as.name('stats'))) %>%
        bind_cols(occurence, stats, motif_pwm)

    return(motif_final)
}



